package com.qmth.paper.library.business.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.paper.library.business.entity.PaperScanTask;
import com.qmth.paper.library.business.service.BusinessCacheService;
import com.qmth.paper.library.business.service.PaperLibraryService;
import com.qmth.paper.library.business.service.PaperScanTaskService;
import com.qmth.paper.library.common.bean.dto.syssetting.SettingObject;
import com.qmth.paper.library.common.bean.dto.syssetting.SimpleObject;
import com.qmth.paper.library.common.contant.SystemConstant;
import com.qmth.paper.library.common.entity.*;
import com.qmth.paper.library.common.enums.ExceptionResultEnum;
import com.qmth.paper.library.common.lock.LockService;
import com.qmth.paper.library.common.lock.LockType;
import com.qmth.paper.library.common.service.*;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.CachePut;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class BusinessCacheServiceImpl implements BusinessCacheService {

    Map<Long, Integer> scanTaskSequenceMap = new HashMap<>();

    @Resource
    PaperScanTaskService paperScanTaskService;

    @Resource
    PaperLibraryService paperLibraryService;

    @Resource
    SysUserService sysUserService;

    @Resource
    LockService lockService;

    @Override
    @Cacheable(value = SystemConstant.CACHE_NAME, key = "'paperScanTask' + #scanTaskId")
    public PaperScanTask getPaperScanTask(Long scanTaskId) {
        return paperScanTaskService.getById(scanTaskId);
    }

    @Override
    @CacheEvict(value = SystemConstant.CACHE_NAME, key = "'paperScanTask' + #scanTaskId")
    public void removeSysSetting(Long schoolId) {

    }

    @Override
    @Cacheable(value = SystemConstant.CACHE_NAME, key = "'sysUser' + #userId")
    public SysUser getSysUser(Long userId) {
        return sysUserService.getById(userId);
    }

    @Override
    public int getSequence(Long paperScanTaskId) {
        try {
            lockService.waitlock(LockType.GET_SEQUENCE, paperScanTaskId);
            if (!scanTaskSequenceMap.containsKey(paperScanTaskId)) {
                scanTaskSequenceMap.put(paperScanTaskId, 1);
                return 1;
            }
            Integer integer = scanTaskSequenceMap.get(paperScanTaskId);
            int value = integer + 1;
            scanTaskSequenceMap.put(paperScanTaskId, value);
            return value;
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("获取自增数失败：" + e.getMessage());
        } finally {
            lockService.unlock(LockType.GET_SEQUENCE, paperScanTaskId);
        }
    }

    @Override
    public void setSequence() {
        QueryWrapper<PaperScanTask> queryWrapper = new QueryWrapper<>();
        List<PaperScanTask> paperScanTaskList = paperScanTaskService.list(queryWrapper);
        for (PaperScanTask paperScanTask : paperScanTaskList) {
            Integer sequence = paperLibraryService.maxSequence(paperScanTask.getId());
            int value = sequence == null ? 0 : sequence.intValue();
            scanTaskSequenceMap.put(paperScanTask.getId(), value);
        }
    }
}
