package com.qmth.paper.library.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.paper.library.business.entity.PaperArchives;
import com.qmth.paper.library.business.entity.PaperLibrary;
import com.qmth.paper.library.business.entity.PaperScanTask;
import com.qmth.paper.library.business.entity.PaperScanTaskDetail;
import com.qmth.paper.library.business.service.*;
import com.qmth.paper.library.common.bean.dto.syssetting.SimpleObject;
import com.qmth.paper.library.common.contant.SysSettingConstant;
import com.qmth.paper.library.common.contant.SystemConstant;
import com.qmth.paper.library.common.entity.BasicStudent;
import com.qmth.paper.library.common.entity.SysPrivilege;
import com.qmth.paper.library.common.entity.SysSetting;
import com.qmth.paper.library.common.entity.SysUser;
import com.qmth.paper.library.common.enums.ExceptionResultEnum;
import com.qmth.paper.library.common.enums.UploadFileEnum;
import com.qmth.paper.library.common.lock.LockService;
import com.qmth.paper.library.common.lock.LockType;
import com.qmth.paper.library.common.service.BasicStudentService;
import com.qmth.paper.library.common.service.CommonCacheService;
import com.qmth.paper.library.common.service.SysPrivilegeService;
import com.qmth.paper.library.common.service.SysSettingService;
import com.qmth.paper.library.common.util.ServletUtil;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.awt.print.Paper;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * <p>
 * 客户端 服务实现类
 * </p>
 */
@Service
public class ClientServiceImpl implements ClientService {

    @Resource
    PaperScanTaskService paperScanTaskService;

    @Resource
    SysPrivilegeService sysPrivilegeService;

    @Resource
    PaperLibraryService paperLibraryService;

    @Resource
    PaperLibraryCommonService paperLibraryCommonService;

    @Resource
    CommonCacheService commonCacheService;

    @Resource
    SysSettingService sysSettingService;

    @Resource
    BasicStudentService basicStudentService;

    @Resource
    PaperScanTaskDetailService paperScanTaskDetailService;

    @Resource
    PaperArchivesService paperArchivesService;

    @Resource
    LockService lockService;

    @Override
    public IPage<PaperScanTask> pageScanTask(Long paperArchivesId, String courseCode, Boolean isMine, Integer pageNumber, Integer pageSize) {
        Long schoolId = SystemConstant.convertIdToLong(ServletUtil.getRequestHeaderSchoolId().toString());
        // 查询任务
        QueryWrapper<PaperScanTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(PaperScanTask::getSchoolId, schoolId);
        if (paperArchivesId != null) {
            queryWrapper.lambda().eq(PaperScanTask::getPaperArchivesId, paperArchivesId);
        }
        if (StringUtils.isNotBlank(courseCode)) {
            queryWrapper.lambda().eq(PaperScanTask::getCourseCode, courseCode);
        }
        if (isMine != null) {
            // 我的任务
            if (isMine) {
                Long userId = Long.valueOf(ServletUtil.getRequestHeaderUserId().toString());
                queryWrapper.lambda().eq(PaperScanTask::getScanUserId, userId);
            } else {
                queryWrapper.lambda().isNull(PaperScanTask::getScanUserId);
            }
        }
        Page<PaperScanTask> taskPage = paperScanTaskService.page(new Page<>(pageNumber, pageSize), queryWrapper);
        Map<Long, String> map = new HashMap<>();
        for (PaperScanTask record : taskPage.getRecords()) {
            if (!map.containsKey(record.getPaperArchivesId())) {
                PaperArchives paperArchives = paperArchivesService.getById(record.getPaperArchivesId());
                record.setArchivesName(paperArchives.getArchivesName());
                map.put(paperArchives.getId(), paperArchives.getArchivesName());
            } else {
                record.setArchivesName(map.get(record.getPaperArchivesId()));
            }

        }
        return taskPage;
    }

    @Override
    public void checkPrivilege(Long userId) {
        List<SysPrivilege> sysPrivileges = sysPrivilegeService.getClientUrlByUserId(userId);
        long count = sysPrivileges.stream().filter(m -> "client".equalsIgnoreCase(m.getUrl())).count();
        if (count == 0) {
            throw ExceptionResultEnum.ERROR.exception("用户没有权限，无法登录");
        }
    }

    @Override
    public boolean bindUser(Long paperScanTaskId, Long userId) {
        try {
            lockService.waitlock(LockType.BIND_SCAN_USER, paperScanTaskId);
            PaperScanTask paperScanTask = paperScanTaskService.getById(paperScanTaskId);
            if (paperScanTask == null) {
                throw ExceptionResultEnum.ERROR.exception("扫描任务不存在");
            }

            if (paperScanTask.getEnable()) {
                throw ExceptionResultEnum.ERROR.exception("扫描任务已关闭");
            }

            // 扫描任务是否被其它人扫描
            if (paperScanTask.getScanUserId() != null && !paperScanTask.getScanUserId().equals(userId)) {
                throw ExceptionResultEnum.ERROR.exception("其它扫描员已开始扫描");
            }

            if (userId == null) {
                SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
                userId = sysUser.getId();
            }

            UpdateWrapper<PaperScanTask> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().set(PaperScanTask::getScanUserId, userId).eq(PaperScanTask::getId, paperScanTaskId);
            return paperScanTaskService.update(updateWrapper);
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("扫描任务错误：" + e.getMessage());
        } finally {
            lockService.unlock(LockType.BIND_SCAN_USER, paperScanTaskId);
        }
    }

    @Transactional
    @Override
    public boolean pictureUpload(Long paperScanTaskId, String studentCode, MultipartFile frontFile, String frontMd5, MultipartFile versoFile, String versoMd5) {
        Long userId = Long.valueOf(ServletUtil.getRequestHeaderUserId().toString());
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());

        if (Objects.isNull(frontFile) || frontFile.getSize() == 0) {
            throw ExceptionResultEnum.ERROR.exception("frontFile（正面文件）为空");
        }
        if (Objects.isNull(versoFile) || versoFile.getSize() == 0) {
            throw ExceptionResultEnum.ERROR.exception("versoFile（背面文件）为空");
        }

        try {
            String frontFileMd5 = DigestUtils.md5Hex(frontFile.getBytes());
            if (!Objects.equals(frontMd5, frontFileMd5)) {
                throw ExceptionResultEnum.MD5_EQUALS_FALSE.exception();
            }
            String versoFileMd5 = DigestUtils.md5Hex(versoFile.getBytes());
            if (!Objects.equals(versoMd5, versoFileMd5)) {
                throw ExceptionResultEnum.MD5_EQUALS_FALSE.exception();
            }

            // 保存图片，生成路径
            Map<String, String> map = paperLibraryCommonService.saveLibraryFile(paperScanTaskId, UploadFileEnum.IMAGE, frontFile, versoFile);

            PaperLibrary paperLibrary = new PaperLibrary();
            paperLibrary.setId(SystemConstant.getDbUuid());
            paperLibrary.setSchoolId(schoolId);
            paperLibrary.setPaperScanTaskId(paperScanTaskId);
            paperLibrary.setPath(map.get("path"));
            paperLibrary.setSequence(Integer.valueOf(map.get("sequence")));
            paperLibrary.setAbnormal(false);
            paperLibrary.setCreateId(userId);
            paperLibrary.setCreateTime(System.currentTimeMillis());

            //开启条码识别，需要自动做一次绑定
            Map<String, SimpleObject> sysSetting = commonCacheService.getSysSetting(schoolId);
            String value = sysSetting.get(SysSettingConstant.BAR_CODE_OPEN).getValue();
            if (sysSetting != null && Boolean.parseBoolean(value)) {
                // 根据准考证号查询学生student_id
                BasicStudent basicStudent = basicStudentService.getBySchoolIdAndStudentCode(studentCode);
                if (basicStudent != null) {
                    // 根据paper_scan_task_id+student_id查询paper_scan_task_detail_id作绑定
                    PaperScanTaskDetail paperScanTaskDetail = paperScanTaskDetailService.getByPaperScanTaskIdAndStudentId(paperScanTaskId, basicStudent.getId());
                    if (paperScanTaskDetail == null) {
                        paperScanTaskDetail = new PaperScanTaskDetail();
                        paperScanTaskDetail.insertInfo(userId);
                        paperScanTaskDetail.setPaperScanTaskId(paperScanTaskId);
                        paperScanTaskDetail.setStudentId(basicStudent.getId());
                        paperScanTaskDetail.setBindCount(0);
                        paperScanTaskDetailService.save(paperScanTaskDetail);
                    }
                    paperLibrary.setPaperScanTaskDetailId(paperScanTaskDetail.getId());
                    paperLibrary.setWordsResult(studentCode);

                    // 绑定数量加1
                    UpdateWrapper<PaperScanTaskDetail> updateWrapper = new UpdateWrapper<>();
                    updateWrapper.lambda().set(PaperScanTaskDetail::getBindCount, paperScanTaskDetail.getBindCount() + 1)
                            .eq(PaperScanTaskDetail::getId, paperScanTaskDetail.getId());
                    paperScanTaskDetailService.update(updateWrapper);
                }
                // 绑定失败，不进异常数据，直接在未处理里，手工绑定
            }
            boolean save = paperLibraryService.save(paperLibrary);
            if (save) {
                // 学生数量
                int studentCount = paperScanTaskDetailService.countStudent(paperScanTaskId);
                // 统计扫描数量
                int scanCount = paperLibraryService.countScanCount(paperScanTaskId);
                int bindCount = paperLibraryService.countBindData(paperScanTaskId);
                // 总扫描数
                Double progress = scanCount == 0 ? 0D :
                        new BigDecimal(bindCount * 100).divide(new BigDecimal(scanCount), BigDecimal.ROUND_DOWN, 2).doubleValue();
                UpdateWrapper<PaperScanTask> updateWrapper = new UpdateWrapper<>();
                updateWrapper.lambda().set(PaperScanTask::getStudentCount, studentCount)
                        .set(PaperScanTask::getScanCount, scanCount)
                        .set(PaperScanTask::getBindCount, bindCount)
                        .set(PaperScanTask::getProgress, progress)
                        .eq(PaperScanTask::getId, paperScanTaskId);
                paperScanTaskService.update(updateWrapper);
            }
            return true;

        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("保存图片失败：" + e.getMessage());
        }
    }

    @Override
    public boolean enable(Long paperScanTaskId, Boolean enable) {
        return paperScanTaskService.enable(paperScanTaskId, enable);
    }
}
