package com.qmth.paper.library.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.qmth.paper.library.business.bean.dto.excel.PaperArchivesCourseImportDto;
import com.qmth.paper.library.business.bean.dto.excel.PaperScanTaskStudentImportDto;
import com.qmth.paper.library.business.bean.result.PaperArchivesDetailResult;
import com.qmth.paper.library.business.bean.result.PaperArchivesResult;
import com.qmth.paper.library.business.bean.result.SelectResult;
import com.qmth.paper.library.business.entity.*;
import com.qmth.paper.library.business.mapper.PaperArchivesMapper;
import com.qmth.paper.library.business.service.PaperArchivesCourseService;
import com.qmth.paper.library.business.service.PaperArchivesService;
import com.qmth.paper.library.business.service.PaperLibraryService;
import com.qmth.paper.library.business.service.PaperScanTaskService;
import com.qmth.paper.library.common.bean.dto.excel.DescribeImportDto;
import com.qmth.paper.library.common.bean.params.DownLoadPaperParams;
import com.qmth.paper.library.common.contant.SystemConstant;
import com.qmth.paper.library.common.entity.BasicStudent;
import com.qmth.paper.library.common.entity.SysUser;
import com.qmth.paper.library.common.enums.ExceptionResultEnum;
import com.qmth.paper.library.common.enums.RoleTypeEnum;
import com.qmth.paper.library.common.service.LibraryCommonService;
import com.qmth.paper.library.common.util.ExcelUtil;
import com.qmth.paper.library.common.util.ServletUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 档案管理表 服务实现类
 * </p>
 */
@Service
public class PaperArchivesServiceImpl extends ServiceImpl<PaperArchivesMapper, PaperArchives> implements PaperArchivesService {

    @Resource
    PaperArchivesCourseService paperArchivesCourseService;

    @Resource
    PaperScanTaskService paperScanTaskService;

    @Resource
    PaperLibraryService paperLibraryService;

    @Resource
    LibraryCommonService libraryCommonService;

    @Override
    public IPage<PaperArchivesResult> pageData(Long semesterId, Long managerOrgId, Long archivesTypeId, String archivesName, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Set<Long> orgIds = libraryCommonService.listSubOrgIds(sysUser.getOrgId());
        return this.baseMapper.pageData(new Page<>(pageNumber, pageSize), schoolId, semesterId, managerOrgId, archivesTypeId, archivesName, orgIds);
    }

    @Override
    public boolean saveData(PaperArchives paperArchives) {
        Long userId = Long.valueOf(ServletUtil.getRequestHeaderUserId().toString());
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());

        // 新增
        if (paperArchives.getId() == null) {
            PaperArchives archives = this.baseMapper.selectByArchivesName(schoolId, paperArchives.getArchivesName());
            if (archives != null) {
                throw ExceptionResultEnum.ERROR.exception("档案名称已存在");
            }
            paperArchives.setId(SystemConstant.getDbUuid());
            paperArchives.setSchoolId(schoolId);
            // 档案编号（时间戳）
            paperArchives.setArchivesCode(String.valueOf(System.currentTimeMillis()));
            paperArchives.setCreateId(userId);
            paperArchives.setCreateTime(System.currentTimeMillis());
            return this.save(paperArchives);
        } else {
            PaperArchives archives = this.baseMapper.selectByArchivesName(schoolId, paperArchives.getArchivesName());
            if (archives != null && !archives.getId().equals(paperArchives.getId())) {
                throw ExceptionResultEnum.ERROR.exception("档案名称已存在");
            }
            PaperArchives oldPaperArchives = this.getById(paperArchives.getId());
            // 档案编号（时间戳）
            oldPaperArchives.setArchivesName(paperArchives.getArchivesName());
            oldPaperArchives.setArchivesTypeId(paperArchives.getArchivesTypeId());
            oldPaperArchives.setSemesterId(paperArchives.getSemesterId());
            oldPaperArchives.setManagerId(paperArchives.getManagerId());
            oldPaperArchives.setManagerOrgId(paperArchives.getManagerOrgId());
            oldPaperArchives.setWarningTime(paperArchives.getWarningTime());
            paperArchives.setUpdateId(userId);
            paperArchives.setUpdateTime(System.currentTimeMillis());
            return this.updateById(paperArchives);
        }
    }

    @Transactional
    @Override
    public boolean delete(String ids) {
        String[] idsArray = ids.split(",");
        for (String id : idsArray) {
            Long idLong = Long.valueOf(id);
            PaperArchives paperArchives = this.getById(idLong);
            if (paperArchives == null) {
                throw ExceptionResultEnum.ERROR.exception("数据有误");
            }

            List<PaperScanTask> paperScanTaskList = paperScanTaskService.listByPaperArchivesId(idLong);
            if (!paperScanTaskList.isEmpty()) {
                throw ExceptionResultEnum.ERROR.exception("已有扫描任务，不能删除");
            }
            this.removeById(idLong);
        }

        return true;
    }

    @Override
    public List<SelectResult> queryManagerOrg(Long semesterId) {
        Long schoolId = SystemConstant.convertIdToLong(String.valueOf(ServletUtil.getRequestHeaderSchoolId()));
        return this.baseMapper.queryManagerOrg(schoolId, semesterId);
    }

    @Override
    public List<SelectResult> queryArchives(Long semesterId) {
        Long schoolId = SystemConstant.convertIdToLong(String.valueOf(ServletUtil.getRequestHeaderSchoolId()));
        QueryWrapper<PaperArchives> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(PaperArchives::getSchoolId, schoolId);
        if (semesterId != null) {
            queryWrapper.lambda().eq(PaperArchives::getSemesterId, semesterId);
        }
        List<PaperArchives> paperArchivesList = this.list(queryWrapper);
        return paperArchivesList.stream().map(m -> {
            SelectResult result = new SelectResult();
            result.setId(String.valueOf(m.getId()));
            result.setCode(m.getArchivesCode());
            result.setName(m.getArchivesName());
            return result;
        }).collect(Collectors.toList());
    }

    @Override
    public IPage<PaperArchivesDetailResult> pageDetail(Long paperArchivesId, Long paperScanTaskId, Long belongOrgId, String majorName, String clazzName, String courseName, String teacherName, String teachClazzName, Boolean isBind, String param, Integer minBindCount, Integer maxBindCount, Integer pageNumber, Integer pageSize) {
        IPage<PaperArchivesDetailResult> detailResultIPage = this.baseMapper.pageDetail(new Page<>(pageNumber, pageSize), paperArchivesId, paperScanTaskId, belongOrgId, majorName, clazzName, courseName, teacherName, teachClazzName, isBind, param, minBindCount, maxBindCount);
        for (PaperArchivesDetailResult record : detailResultIPage.getRecords()) {
            QueryWrapper<PaperLibrary> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(PaperLibrary::getPaperScanTaskDetailId, record.getPaperScanTaskDetailId());
            int count = paperLibraryService.count(queryWrapper);
            record.setBindCount(count);
        }
        return detailResultIPage;
    }

    @Override
    public List<PaperArchivesDetailResult> listDetail(DownLoadPaperParams downLoadPaperParams) {
        String ids = downLoadPaperParams.getIds();
        List<PaperArchivesDetailResult> paperArchivesDetailResults;
        if (StringUtils.isNotBlank(ids) && ids.split(",").length > 0) {
            String[] idsArray = ids.split(",");
            paperArchivesDetailResults = this.baseMapper.listDetail(Arrays.asList(idsArray));
        } else {
            Long paperArchivesId = downLoadPaperParams.getPaperArchivesId();
            if (paperArchivesId == null) {
                throw ExceptionResultEnum.ERROR.exception("档案ID必传");
            }
            Long belongOrgId = downLoadPaperParams.getBelongOrgId();
            String majorName = downLoadPaperParams.getMajorName();
            String clazzName = downLoadPaperParams.getClazzName();
            String courseName = downLoadPaperParams.getCourseName();
            String teacherName = downLoadPaperParams.getTeacherName();
            String teachClazzName = downLoadPaperParams.getTeachClazzName();
            Boolean isBind = downLoadPaperParams.getIsBind() == null ? null : downLoadPaperParams.getIsBind() == 1;
            String param = downLoadPaperParams.getParam();
            paperArchivesDetailResults = this.baseMapper.pageDetail(paperArchivesId, belongOrgId, majorName, clazzName, courseName, teacherName, teachClazzName, isBind, param);
        }
        for (PaperArchivesDetailResult record : paperArchivesDetailResults) {
            QueryWrapper<PaperLibrary> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(PaperLibrary::getPaperScanTaskDetailId, record.getPaperScanTaskDetailId());
            int count = paperLibraryService.count(queryWrapper);
            record.setBindCount(count);
        }
        return paperArchivesDetailResults;
    }

    @Override
    public List<SelectResult> queryManagerUser() {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        return this.baseMapper.queryManagerUser(schoolId, RoleTypeEnum.ARCHIVIST.name());
    }

    @Transactional
    @Override
    public void importData(Long paperArchivesId, MultipartFile file) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        try {
            List<LinkedMultiValueMap<Integer, Object>> finalList = ExcelUtil.excelReader(file.getInputStream(), Lists.newArrayList(PaperArchivesCourseImportDto.class, DescribeImportDto.class), (finalExcelList, finalColumnNameList, finalExcelErrorList) -> finalExcelList);
            List<PaperArchivesCourseImportDto> datasource = new ArrayList<>();
            for (int i = 0; i < finalList.size(); i++) {
                LinkedMultiValueMap<Integer, Object> excelMap = finalList.get(i);
                List<Object> paperArchivesCourseImportDtoList = excelMap.get(i);
                if (CollectionUtils.isEmpty(paperArchivesCourseImportDtoList) || paperArchivesCourseImportDtoList.get(0) instanceof DescribeImportDto) {
                    continue;
                }
                // 类型转换
                if (paperArchivesCourseImportDtoList.get(0) instanceof PaperArchivesCourseImportDto) {
                    datasource = paperArchivesCourseImportDtoList.stream().map(e -> {
                        PaperArchivesCourseImportDto paperArchivesCourseImportDto = new PaperArchivesCourseImportDto();
                        BeanUtils.copyProperties(e, paperArchivesCourseImportDto);
                        return paperArchivesCourseImportDto;
                    }).collect(Collectors.toList());
                }
            }
            if (!CollectionUtils.isEmpty(datasource)) {
                Map<String, PaperArchivesCourseImportDto> courseImportDtoMap = new HashMap<>();
                for (PaperArchivesCourseImportDto paperArchivesCourseImportDto : datasource) {
                    String courseCode = paperArchivesCourseImportDto.getCourseCode();
                    String courseName = paperArchivesCourseImportDto.getCourseName();
                    String teacherName = paperArchivesCourseImportDto.getTeacherName();
                    String teachClazzName = paperArchivesCourseImportDto.getTeachClazzName();

                    if (courseImportDtoMap.containsKey(courseCode)) {
                        // 如果试卷任务map中包含考生 - 检验其他信息是否一致(姓名，班级)
                        PaperArchivesCourseImportDto v = courseImportDtoMap.get(courseCode);
                        String vCourseName = v.getCourseName();

                        if (!courseName.equals(vCourseName)) {
                            throw ExceptionResultEnum.ERROR.exception("课程代码[" + courseCode + "]存在不同的课程名称[" + vCourseName + "," + courseName + "]");
                        }

                        String vTeacherName = v.getTeacherName();
                        if (!teacherName.equals(vTeacherName)) {
                            throw ExceptionResultEnum.ERROR.exception("课程代码[" + courseCode + "]存在不同的任课老师[" + vTeacherName + "," + teacherName + "]");
                        }

                        String vTeachClazzName = v.getTeachClazzName();
                        if (!teachClazzName.equals(vTeachClazzName)) {
                            throw ExceptionResultEnum.ERROR.exception("课程代码[" + courseCode + "]存在不同的教学班[" + vTeachClazzName + "," + teachClazzName + "]");
                        }
                        // 校验通过，删除掉重复数据
                        datasource.remove(paperArchivesCourseImportDto);
                    } else {
                        // 没包含直接put
                        courseImportDtoMap.put(courseCode, paperArchivesCourseImportDto);
                    }
                }

                // 学生基础数据集合
                List<PaperArchivesCourse> paperArchivesCourses = paperArchivesCourseService.list(new QueryWrapper<PaperArchivesCourse>().lambda().eq(PaperArchivesCourse::getPaperArchivesId, paperArchivesId));
                List<PaperArchivesCourse> paperArchivesCourseList = new ArrayList<>();
                for (PaperArchivesCourseImportDto e : datasource) {
                    String courseCode = e.getCourseCode();
                    String courseName = e.getCourseName();
                    String teacherName = e.getTeacherName();
                    String teachClazzName = e.getTeachClazzName();
                    List<PaperArchivesCourse> archivesCourses = paperArchivesCourses.stream().filter(s -> courseCode.equals(s.getCourseCode())).collect(Collectors.toList());
                    PaperArchivesCourse paperArchivesCourse;
                    if (archivesCourses.isEmpty()) {
                        paperArchivesCourse = new PaperArchivesCourse(schoolId, paperArchivesId, courseCode, courseName, teacherName, teachClazzName);
                        paperArchivesCourse.insertInfo(sysUser.getId());
                    } else if (archivesCourses.size() == 1) {
                        paperArchivesCourse = archivesCourses.get(0);

                        if (!courseName.equals(paperArchivesCourse.getCourseName())) {
                            throw ExceptionResultEnum.ERROR.exception("课程代码[" + courseCode + "]已存在的课程名称为[" + paperArchivesCourse.getCourseName() + "]");
                        }
                        paperArchivesCourse.setTeacherName(teacherName);
                        paperArchivesCourse.setTeachClazzName(teachClazzName);
                        paperArchivesCourse.updateInfo(sysUser.getId());
                    } else {
                        throw ExceptionResultEnum.ERROR.exception("课程代码[" + courseCode + "]查出多条数据");
                    }

                    paperArchivesCourseList.add(paperArchivesCourse);
                }
                if (!CollectionUtils.isEmpty(paperArchivesCourseList)) {
                    paperArchivesCourseService.saveOrUpdateBatch(paperArchivesCourseList);
                }
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        } catch (NoSuchFieldException e) {
            throw ExceptionResultEnum.ERROR.exception("请选择导入文件");
        }
    }

}
